#!/usr/bin/env python
# CESYS Software License - Version 1.0 - January 1st, 2016
#
# Permission is hereby granted, free of charge, to any person or organization
# obtaining a copy of the software and accompanying documentation covered by
# this license (the "Software") to use, reproduce, display, distribute,
# execute, and transmit the Software, and to prepare derivative works of the
# Software, and to permit third-parties to whom the Software is furnished to
# do so, all subject to the following:
#
# The software must only be used to operate hardware manufactured by 
# CESYS GmbH, Herzogenaurach.
#
# The copyright notices in the Software and this entire statement, including
# the above license grant, this restriction and the following disclaimer,
# must be included in all copies of the Software, in whole or in part, and
# all derivative works of the Software, unless such copies or derivative
# works are solely in the form of machine-executable object code generated by
# a source language processor.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
# SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
# FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# 
# Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
from CeboMsrApiPython import LibraryInterface, DeviceType
from math import sin

# CeboMsr - Python example - Frame I/O.
#
# Multiple inputs are read multiple times in a single transaction,
# values are printed to the console.
# AO-1 and DP-2 are set multiple times in a single transaction.
#
# Cebo-Stick example does input transfer only.

def frameIoTestLC(device):
    """
    Do frame IO (read/write multiple I/O in single calls).
    device: Device to use for test.
    """
    # Prepare and fill the list of inputs to read.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDifferentialInputs()[1],
        device.getDigitalPorts()[0],
        device.getDigitalPorts()[1],
        device.getCounters()[0]
    ]

    # Setup device with this selection.
    device.setupInputFrame(inputs)

    # Read the values multiple times and write them to the console.
    for _ in range(100):
        # Read all inputs into the instance of InputFrame.
        inFrame = device.readFrame()

        # Write results to the console.
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("DigitalPort #1: %d" % inFrame.getDigitalPort(1)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V, " +
            ("Differential #1: %.2f" % inFrame.getDifferential(1)) + " V, " +
            ("Counter #0: %d" % inFrame.getCounter(0)))

    # Write to analog out #1 and digital out #2 in one call.
    # Prepare and set output selection.
    outputs = [
        device.getDigitalPorts()[2],
        device.getAnalogOutputs()[1]
    ]

    # Prepare device ...
    device.setupOutputFrame(outputs)

    # Prepare digital port #2 as output.
    device.getDigitalPorts()[2].setOutputEnableMask(0xf)

    # Create instance of OutputFrame. There's no direct construction, as this
    # instance may vary between different device types.
    outFrame = device.createOutputFrame()

    # Write it to hardware, modify contents in every looping.
    for i in range(100):
        outFrame.setAnalogOutput(1, float(3 * sin(float(i) / 100)))
        outFrame.setDigitalPort(2, i)

        device.writeFrame(outFrame)

    # Reset to power up defaults.
    device.resetDevice()

def frameIoTestStick(device):
    """
    Do frame IO (read/write multiple I/O in single calls).
    device: Device to use for test.
    """
    # Prepare and fill the list of inputs to read.
    inputs = [
        device.getSingleEndedInputs()[0],
        device.getSingleEndedInputs()[1],
        device.getDigitalPorts()[0],
        device.getCounters()[0]
    ]

    # Setup device with this selection.
    device.setupInputFrame(inputs)

    # Read the values multiple times and write them to the console.
    for _ in range(100):
        # Read all inputs into the instance of InputFrame.
        inFrame = device.readFrame()

        # Write results to the console.
        print(
            ("DigitalPort #0: %d" % inFrame.getDigitalPort(0)) + ", " +
            ("SingleEnded #0: %.2f" % inFrame.getSingleEnded(0)) + " V, " +
            ("SingleEnded #1: %.2f" % inFrame.getSingleEnded(1)) + " V, " +
            ("Counter #0: %d" % inFrame.getCounter(0)))

    # Reset to power up defaults.
    device.resetDevice()

def main():
    """
    The examples main method.
    """
    try:
        # Search for devices ...
        devices = LibraryInterface.enumerate(DeviceType.All)

        # If at least one has been found, use the first one ...
        if (len(devices) > 0):
            device = devices[0]

            # Open device, nothing can be done without doing this.
            device.open()

            # *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
            # *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
            # *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
            if (device.getDeviceType() == DeviceType.CeboLC):
                frameIoTestLC(device)
            elif (device.getDeviceType() == DeviceType.CeboStick):
                frameIoTestStick(device)
            else:
                raise SystemError("No supported device detected.")

            # Finalize device usage, this free's up the device, so it can be used
            # again, including other applications.
            device.close()
    except Exception as e:
        print(e)

if (__name__ == "__main__"):
    main()
